# Generated by cpp11: do not edit by hand

collectorGuess <- function(input, locale_, guessInteger) {
  .Call("_readr_collectorGuess", input, locale_, guessInteger, PACKAGE = "readr")
}

read_connection_ <- function(con, filename, chunk_size) {
  .Call("_readr_read_connection_", con, filename, chunk_size, PACKAGE = "readr")
}

utctime_ <- function(year, month, day, hour, min, sec, psec) {
  .Call("_readr_utctime_", year, month, day, hour, min, sec, psec, PACKAGE = "readr")
}

dim_tokens_ <- function(sourceSpec, tokenizerSpec) {
  .Call("_readr_dim_tokens_", sourceSpec, tokenizerSpec, PACKAGE = "readr")
}

count_fields_ <- function(sourceSpec, tokenizerSpec, n_max) {
  .Call("_readr_count_fields_", sourceSpec, tokenizerSpec, n_max, PACKAGE = "readr")
}

guess_header_ <- function(sourceSpec, tokenizerSpec, locale_) {
  .Call("_readr_guess_header_", sourceSpec, tokenizerSpec, locale_, PACKAGE = "readr")
}

tokenize_ <- function(sourceSpec, tokenizerSpec, n_max) {
  .Call("_readr_tokenize_", sourceSpec, tokenizerSpec, n_max, PACKAGE = "readr")
}

parse_vector_ <- function(x, collectorSpec, locale_, na, trim_ws) {
  .Call("_readr_parse_vector_", x, collectorSpec, locale_, na, trim_ws, PACKAGE = "readr")
}

read_file_ <- function(sourceSpec, locale_) {
  .Call("_readr_read_file_", sourceSpec, locale_, PACKAGE = "readr")
}

read_file_raw_ <- function(sourceSpec) {
  .Call("_readr_read_file_raw_", sourceSpec, PACKAGE = "readr")
}

read_lines_ <- function(sourceSpec, locale_, na, n_max, skip_empty_rows, progress) {
  .Call("_readr_read_lines_", sourceSpec, locale_, na, n_max, skip_empty_rows, progress, PACKAGE = "readr")
}

read_lines_chunked_ <- function(sourceSpec, locale_, na, chunkSize, callback, skip_empty_rows, progress) {
  invisible(.Call("_readr_read_lines_chunked_", sourceSpec, locale_, na, chunkSize, callback, skip_empty_rows, progress, PACKAGE = "readr"))
}

read_lines_raw_ <- function(sourceSpec, n_max, progress) {
  .Call("_readr_read_lines_raw_", sourceSpec, n_max, progress, PACKAGE = "readr")
}

read_lines_raw_chunked_ <- function(sourceSpec, chunkSize, callback, progress) {
  invisible(.Call("_readr_read_lines_raw_chunked_", sourceSpec, chunkSize, callback, progress, PACKAGE = "readr"))
}

read_tokens_ <- function(sourceSpec, tokenizerSpec, colSpecs, colNames, locale_, n_max, progress) {
  .Call("_readr_read_tokens_", sourceSpec, tokenizerSpec, colSpecs, colNames, locale_, n_max, progress, PACKAGE = "readr")
}

read_tokens_chunked_ <- function(sourceSpec, callback, chunkSize, tokenizerSpec, colSpecs, colNames, locale_, progress) {
  invisible(.Call("_readr_read_tokens_chunked_", sourceSpec, callback, chunkSize, tokenizerSpec, colSpecs, colNames, locale_, progress, PACKAGE = "readr"))
}

melt_tokens_ <- function(sourceSpec, tokenizerSpec, colSpecs, locale_, n_max, progress) {
  .Call("_readr_melt_tokens_", sourceSpec, tokenizerSpec, colSpecs, locale_, n_max, progress, PACKAGE = "readr")
}

melt_tokens_chunked_ <- function(sourceSpec, callback, chunkSize, tokenizerSpec, colSpecs, locale_, progress) {
  invisible(.Call("_readr_melt_tokens_chunked_", sourceSpec, callback, chunkSize, tokenizerSpec, colSpecs, locale_, progress, PACKAGE = "readr"))
}

guess_types_ <- function(sourceSpec, tokenizerSpec, locale_, n) {
  .Call("_readr_guess_types_", sourceSpec, tokenizerSpec, locale_, n, PACKAGE = "readr")
}

whitespaceColumns <- function(sourceSpec, n, comment) {
  .Call("_readr_whitespaceColumns", sourceSpec, n, comment, PACKAGE = "readr")
}

type_convert_col <- function(x, spec, locale_, col, na, trim_ws) {
  .Call("_readr_type_convert_col", x, spec, locale_, col, na, trim_ws, PACKAGE = "readr")
}

stream_delim_ <- function(df, connection, delim, na, col_names, bom, quote_escape, eol) {
  .Call("_readr_stream_delim_", df, connection, delim, na, col_names, bom, quote_escape, eol, PACKAGE = "readr")
}

write_lines_ <- function(lines, connection, na, sep) {
  invisible(.Call("_readr_write_lines_", lines, connection, na, sep, PACKAGE = "readr"))
}

write_lines_raw_ <- function(x, connection, sep) {
  invisible(.Call("_readr_write_lines_raw_", x, connection, sep, PACKAGE = "readr"))
}

write_file_ <- function(x, connection) {
  invisible(.Call("_readr_write_file_", x, connection, PACKAGE = "readr"))
}

write_file_raw_ <- function(x, connection) {
  invisible(.Call("_readr_write_file_raw_", x, connection, PACKAGE = "readr"))
}
