#ifndef FASTREAD_BOOST_H_
#define FASTREAD_BOOST_H_

#pragma GCC system_header

#include <boost/interprocess/file_mapping.hpp>
#include <boost/interprocess/mapped_region.hpp>

#include <boost/container/string.hpp>
#include <boost/noncopyable.hpp>
#include <boost/shared_ptr.hpp>

#include <boost/algorithm/string/predicate.hpp>
#include <boost/spirit/include/qi.hpp>

#endif
